@@ -28,7 +28,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 #include "driver_interface.h"
 #include "JackLibGlobals.h"
 
-
 #include <math.h>
 #include <string>
 #include <algorithm>
@@ -40,9 +39,6 @@ namespace Jack
 
 #define IsRealTime() ((fProcess != NULL) | (fThreadFun != NULL) | (fSync != NULL) | (fTimebase != NULL))
 
-JackClient::JackClient():fThread(this)
-{}
-
 JackClient::JackClient(JackSynchro* table):fThread(this)
 {
     fSynchroTable = table;
@@ -63,6 +59,7 @@ JackClient::JackClient(JackSynchro* table):fThread(this)
     fThreadFun = NULL;
     fSession = NULL;
     fLatency = NULL;
+    fPropertyChange = NULL;
 
     fProcessArg = NULL;
     fGraphOrderArg = NULL;
@@ -81,28 +78,46 @@ JackClient::JackClient(JackSynchro* table):fThread(this)
     fThreadFunArg = NULL;
     fSessionArg = NULL;
     fLatencyArg = NULL;
+    fPropertyChangeArg = NULL;
+
+    fSessionReply = kPendingSessionReply;
 }
 
 JackClient::~JackClient()
 {}
 
+void JackClient::ShutDown(jack_status_t code, const char* message)
+{
+    jack_log("JackClient::ShutDown");
+ 
+    // If "fInfoShutdown" callback, then call it
+    if (fInfoShutdown) {
+        fInfoShutdown(code, message, fInfoShutdownArg);
+        fInfoShutdown = NULL;
+    // Otherwise possibly call the normal "fShutdown"
+    } else if (fShutdown) {
+        fShutdown(fShutdownArg);
+        fShutdown = NULL;
+    }
+}
+
 int JackClient::Close()
 {
     jack_log("JackClient::Close ref = %ld", GetClientControl()->fRefNum);
     int result = 0;
 
     Deactivate();
-    fChannel->Stop();  // Channels is stopped first to avoid receiving notifications while closing
-
-    // Request close only if server is still running
-    if (JackGlobals::fServerRunning) {
-        fChannel->ClientClose(GetClientControl()->fRefNum, &result);
-    } else {
-        jack_log("JackClient::Close server is shutdown");
-    }
-
+    
+    // Channels is stopped first to avoid receiving notifications while closing
+    fChannel->Stop();  
+    // Then close client
+    fChannel->ClientClose(GetClientControl()->fRefNum, &result);
+  
     fChannel->Close();
+    assert(JackGlobals::fSynchroMutex);
+    JackGlobals::fSynchroMutex->Lock();
     fSynchroTable[GetClientControl()->fRefNum].Disconnect();
+    JackGlobals::fSynchroMutex->Unlock();
     JackGlobals::fClientTable[GetClientControl()->fRefNum] = NULL;
     return result;
 }
@@ -131,8 +146,9 @@ void JackClient::SetupDriverSync(bool freewheel)
         }
     } else {
         jack_log("JackClient::SetupDriverSync driver sem in normal mode");
-        for (int i = 0; i < GetEngineControl()->fDriverNum; i++)
+        for (int i = 0; i < GetEngineControl()->fDriverNum; i++) {
             fSynchroTable[i].SetFlush(false);
+        }
     }
 }
 
@@ -149,6 +165,8 @@ int JackClient::ClientNotify(int refnum, const char* name, int notify, int sync,
 {
     int res = 0;
 
+    jack_log("JackClient::ClientNotify ref = %ld name = %s notify = %ld", refnum, name, notify);
+
     // Done all time: redirected on subclass implementation JackLibClient and JackInternalClient
     switch (notify) {
 
@@ -212,7 +230,10 @@ int JackClient::ClientNotify(int refnum, const char* name, int notify, int sync,
             case kStartFreewheelCallback:
                 jack_log("JackClient::kStartFreewheel");
                 SetupDriverSync(true);
-                fThread.DropRealTime();     // Always done (JACK server in RT mode or not...)
+                // Drop RT only when the RT thread is actually running
+                if (fThread.GetStatus() == JackThread::kRunning) {
+                    fThread.DropRealTime();     
+                }
                 if (fFreewheel) {
                     fFreewheel(1, fFreewheelArg);
                 }
@@ -224,8 +245,9 @@ int JackClient::ClientNotify(int refnum, const char* name, int notify, int sync,
                 if (fFreewheel) {
                     fFreewheel(0, fFreewheelArg);
                 }
-                if (GetEngineControl()->fRealTime) {
-                    if (fThread.AcquireRealTime() < 0) {
+                // Acquire RT only when the RT thread is actually running
+                if (GetEngineControl()->fRealTime && fThread.GetStatus() == JackThread::kRunning) {
+                    if (fThread.AcquireRealTime(GetEngineControl()->fClientPriority) < 0) {
                         jack_error("JackClient::AcquireRealTime error");
                     }
                 }
@@ -275,32 +297,41 @@ int JackClient::ClientNotify(int refnum, const char* name, int notify, int sync,
 
             case kShutDownCallback:
                 jack_log("JackClient::kShutDownCallback");
-                if (fInfoShutdown) {
-                    fInfoShutdown((jack_status_t)value1, message, fInfoShutdownArg);
-                    fInfoShutdown = NULL;
-                }
+                ShutDown(jack_status_t(value1), message);
                 break;
 
             case kSessionCallback:
                 jack_log("JackClient::kSessionCallback");
                 if (fSession) {
-                    jack_session_event_t *event = (jack_session_event_t *) malloc( sizeof(jack_session_event_t) );
-                    char uuid_buf[JACK_UUID_SIZE];
-                    event->type = (jack_session_event_type_t) value1;
-                    event->session_dir = strdup( message );
+                    jack_session_event_t* event = (jack_session_event_t*)malloc( sizeof(jack_session_event_t));
+                    char uuid_buf[JACK_UUID_STRING_SIZE];
+                    event->type = (jack_session_event_type_t)value1;
+                    event->session_dir = strdup(message);
                     event->command_line = NULL;
-                    event->flags = (jack_session_flags_t) 0;
-                    snprintf( uuid_buf, sizeof(uuid_buf), "%d", GetClientControl()->fSessionID );
-                    event->client_uuid = strdup( uuid_buf );
-                    fImmediateSessionReply = false;
+                    event->flags = (jack_session_flags_t)0;
+                    jack_uuid_unparse(GetClientControl()->fSessionID, uuid_buf);
+                    event->client_uuid = strdup(uuid_buf);
+                    fSessionReply = kPendingSessionReply;
+                    // Session callback may change fSessionReply by directly using jack_session_reply
                     fSession(event, fSessionArg);
-                    res = (fImmediateSessionReply) ? 1 : 2;
+                    res = fSessionReply;
                 }
                 break;
 
             case kLatencyCallback:
                 res = HandleLatencyCallback(value1);
                 break;
+
+            case kPropertyChangeCallback: {
+                jack_uuid_t subject;
+                jack_uuid_parse(name, &subject);
+                const char* key = message;
+                jack_property_change_t change = (jack_property_change_t)value1;
+                jack_log("JackClient::kPropertyChangeCallback subject = %x key = %s change = %x", subject, key, change);
+                if (fPropertyChange)
+                    fPropertyChange(subject, key, change, fPropertyChangeArg);
+                break;
+            }
         }
     }
 
@@ -318,7 +349,7 @@ int JackClient::HandleLatencyCallback(int status)
     list<jack_port_id_t>::iterator it;
 
 	for (it = fPortList.begin(); it != fPortList.end(); it++) {
-	   JackPort* port = GetGraphManager()->GetPort(*it);
+        JackPort* port = GetGraphManager()->GetPort(*it);
         if ((port->GetFlags() & JackPortIsOutput) && (mode == JackPlaybackLatency)) {
             GetGraphManager()->RecalculateLatency(*it, mode);
 		}
@@ -341,15 +372,18 @@ int JackClient::HandleLatencyCallback(int status)
                 if (port->GetFlags() & JackPortIsOutput) {
 					jack_latency_range_t other_latency;
 					port->GetLatencyRange(mode, &other_latency);
-					if (other_latency.max > latency.max)
+					if (other_latency.max > latency.max) {
 						latency.max = other_latency.max;
-					if (other_latency.min < latency.min)
+                    }
+					if (other_latency.min < latency.min) {
 						latency.min = other_latency.min;
+                    }
 				}
 			}
 
-			if (latency.min == UINT32_MAX)
+			if (latency.min == UINT32_MAX) {
 				latency.min = 0;
+            }
 
 			/* now set the found latency on all input ports
 			 */
@@ -368,15 +402,18 @@ int JackClient::HandleLatencyCallback(int status)
 				if (port->GetFlags() & JackPortIsInput) {
 					jack_latency_range_t other_latency;
                     port->GetLatencyRange(mode, &other_latency);
-					if (other_latency.max > latency.max)
+					if (other_latency.max > latency.max) {
 						latency.max = other_latency.max;
-					if (other_latency.min < latency.min)
+                    }
+					if (other_latency.min < latency.min) {
 						latency.min = other_latency.min;
+                    }
 				}
 			}
 
-			if (latency.min == UINT32_MAX)
+			if (latency.min == UINT32_MAX) {
 				latency.min = 0;
+            }
 
 			/* now set the found latency on all output ports
 			 */
@@ -404,13 +441,15 @@ connected to the client may not be activated.
 int JackClient::Activate()
 {
     jack_log("JackClient::Activate");
-    if (IsActive())
+    if (IsActive()) {
         return 0;
+    }
 
     // RT thread is started only when needed...
     if (IsRealTime()) {
-        if (StartThread() < 0)
+        if (StartThread() < 0) {
             return -1;
+        }
     }
 
     /*
@@ -435,8 +474,9 @@ int JackClient::Activate()
 int JackClient::Deactivate()
 {
     jack_log("JackClient::Deactivate");
-    if (!IsActive())
+    if (!IsActive()) {
         return 0;
+    }
 
     GetClientControl()->fActive = false;
 
@@ -450,8 +490,9 @@ int JackClient::Deactivate()
     jack_log("JackClient::Deactivate res = %ld", result);
 
     // RT thread is stopped only when needed...
-    if (IsRealTime())
+    if (IsRealTime()) {
         fThread.Kill();
+    }
     return result;
 }
 
@@ -488,25 +529,22 @@ bool JackClient::Init()
     InitAux();
 
     // Setup context
-    if (!jack_tls_set(JackGlobals::fRealTime, this))
-        jack_error("failed to set thread realtime key");
-
-    if (GetEngineControl()->fRealTime)
-        set_threaded_log_function();
+    if (!jack_tls_set(JackGlobals::fRealTimeThread, this)) {
+        jack_error("Failed to set thread realtime key");
+    }
 
     // Setup RT
     if (GetEngineControl()->fRealTime) {
-        if (fThread.AcquireSelfRealTime(GetEngineControl()->fClientPriority) < 0) {
-            jack_error("JackClient::AcquireSelfRealTime error");
-        }
+        set_threaded_log_function();
+        SetupRealTime();
     }
 
     return true;
 }
 
-int JackClient::StartThread()
+void JackClient::SetupRealTime()
 {
-    jack_log("JackClient::StartThread : period = %ld computation = %ld constraint = %ld",
+    jack_log("JackClient::Init : period = %ld computation = %ld constraint = %ld",
              long(int64_t(GetEngineControl()->fPeriod) / 1000.0f),
              long(int64_t(GetEngineControl()->fComputation) / 1000.0f),
              long(int64_t(GetEngineControl()->fConstraint) / 1000.0f));
@@ -514,6 +552,13 @@ int JackClient::StartThread()
     // Will do "something" on OSX only...
     fThread.SetParams(GetEngineControl()->fPeriod, GetEngineControl()->fComputation, GetEngineControl()->fConstraint);
 
+    if (fThread.AcquireSelfRealTime(GetEngineControl()->fClientPriority) < 0) {
+        jack_error("JackClient::AcquireSelfRealTime error");
+    }
+}
+
+int JackClient::StartThread()
+{
     if (fThread.StartSync() < 0) {
         jack_error("Start thread error");
         return -1;
@@ -555,19 +600,22 @@ inline void JackClient::ExecuteThread()
 
 inline jack_nframes_t JackClient::CycleWaitAux()
 {
-    if (!WaitSync())
+    if (!WaitSync()) {
         Error();   // Terminates the thread
+    }
     CallSyncCallbackAux();
     return GetEngineControl()->fBufferSize;
 }
 
 inline void JackClient::CycleSignalAux(int status)
 {
-    if (status == 0)
+    if (status == 0) {
         CallTimebaseCallbackAux();
+    }
     SignalSync();
-    if (status != 0)
+    if (status != 0) {
         End();     // Terminates the thread
+    }
 }
 
 jack_nframes_t JackClient::CycleWait()
@@ -623,7 +671,7 @@ inline void JackClient::Error()
     fThread.DropSelfRealTime();
     GetClientControl()->fActive = false;
     fChannel->ClientDeactivate(GetClientControl()->fRefNum, &result);
-    ShutDown();
+    ShutDown(jack_status_t(JackFailure | JackServerError), JACK_SERVER_FAILURE);
     fThread.Terminate();
 }
 
@@ -634,15 +682,15 @@ inline void JackClient::Error()
 int JackClient::PortRegister(const char* port_name, const char* port_type, unsigned long flags, unsigned long buffer_size)
 {
     // Check if port name is empty
-    string port_name_str = string(port_name);
-    if (port_name_str.size() == 0) {
+    string port_short_name_str = string(port_name);
+    if (port_short_name_str.size() == 0) {
         jack_error("port_name is empty");
         return 0; // Means failure here...
     }
 
     // Check port name length
-    string name = string(GetClientControl()->fName) + string(":") + port_name_str;
-    if (name.size() >= JACK_PORT_NAME_SIZE) {
+    string port_full_name_str = string(GetClientControl()->fName) + string(":") + port_short_name_str;
+    if (port_full_name_str.size() >= REAL_JACK_PORT_NAME_SIZE) {
         jack_error("\"%s:%s\" is too long to be used as a JACK port name.\n"
                    "Please use %lu characters or less",
                    GetClientControl()->fName,
@@ -653,10 +701,10 @@ int JackClient::PortRegister(const char* port_name, const char* port_type, unsig
 
     int result = -1;
     jack_port_id_t port_index = NO_PORT;
-    fChannel->PortRegister(GetClientControl()->fRefNum, name.c_str(), port_type, flags, buffer_size, &port_index, &result);
+    fChannel->PortRegister(GetClientControl()->fRefNum, port_full_name_str.c_str(), port_type, flags, buffer_size, &port_index, &result);
 
     if (result == 0) {
-        jack_log("JackClient::PortRegister ref = %ld name = %s type = %s port_index = %ld", GetClientControl()->fRefNum, name.c_str(), port_type, port_index);
+        jack_log("JackClient::PortRegister ref = %ld name = %s type = %s port_index = %ld", GetClientControl()->fRefNum, port_full_name_str.c_str(), port_type, port_index);
         fPortList.push_back(port_index);
         return port_index;
     } else {
@@ -683,6 +731,14 @@ int JackClient::PortUnRegister(jack_port_id_t port_index)
 int JackClient::PortConnect(const char* src, const char* dst)
 {
     jack_log("JackClient::Connect src = %s dst = %s", src, dst);
+    if (strlen(src) >= REAL_JACK_PORT_NAME_SIZE) {
+        jack_error("\"%s\" is too long to be used as a JACK port name.\n", src);
+        return -1; 
+    }
+    if (strlen(dst) >= REAL_JACK_PORT_NAME_SIZE) {
+        jack_error("\"%s\" is too long to be used as a JACK port name.\n", dst);
+        return -1; 
+    }
     int result = -1;
     fChannel->PortConnect(GetClientControl()->fRefNum, src, dst, &result);
     return result;
@@ -691,6 +747,14 @@ int JackClient::PortConnect(const char* src, const char* dst)
 int JackClient::PortDisconnect(const char* src, const char* dst)
 {
     jack_log("JackClient::Disconnect src = %s dst = %s", src, dst);
+    if (strlen(src) >= REAL_JACK_PORT_NAME_SIZE) {
+        jack_error("\"%s\" is too long to be used as a JACK port name.\n", src);
+        return -1; 
+    }
+    if (strlen(dst) >= REAL_JACK_PORT_NAME_SIZE) {
+        jack_error("\"%s\" is too long to be used as a JACK port name.\n", dst);
+        return -1; 
+    }
     int result = -1;
     fChannel->PortDisconnect(GetClientControl()->fRefNum, src, dst, &result);
     return result;
@@ -742,27 +806,6 @@ int JackClient::ComputeTotalLatencies()
     return result;
 }
 
-/*
-ShutDown is called:
-- from the RT thread when Execute method fails
-- possibly from a "closed" notification channel
-(Not needed since the synch object used (Sema of Fifo will fails when server quits... see ShutDown))
-*/
-
-void JackClient::ShutDown()
-{
-    jack_log("JackClient::ShutDown");
-    JackGlobals::fServerRunning = false;
-
-    if (fInfoShutdown) {
-        fInfoShutdown(JackFailure, "JACK server has been closed", fInfoShutdownArg);
-        fInfoShutdown = NULL;
-    } else if (fShutdown) {
-        fShutdown(fShutdownArg);
-        fShutdown = NULL;
-    }
-}
-
 //----------------------
 // Transport management
 //----------------------
@@ -775,8 +818,9 @@ inline int JackClient::ActivateAux()
         jack_log("JackClient::ActivateAux");
 
         // RT thread is started
-        if (StartThread() < 0)
+        if (StartThread() < 0) {
             return -1;
+        }
 
         int result = -1;
         GetClientControl()->fCallback[kRealTimeCallback] = IsRealTime();
@@ -822,7 +866,7 @@ int JackClient::SetTimebaseCallback(int conditional, JackTimebaseCallback timeba
     } else {
         fTimebase = NULL;
         fTimebaseArg = NULL;
-        return -1;
+        return result;
     }
 }
 
@@ -843,14 +887,14 @@ void JackClient::TransportLocate(jack_nframes_t frame)
     GetEngineControl()->fTransport.RequestNewPos(&pos);
 }
 
-int JackClient::TransportReposition(jack_position_t* pos)
+int JackClient::TransportReposition(const jack_position_t* pos)
 {
     jack_position_t tmp = *pos;
     jack_log("JackClient::TransportReposition pos = %ld", pos->frame);
     if (tmp.valid & ~JACK_POSITION_MASK) {
         return EINVAL;
     } else {
-        GetEngineControl()->fTransport.RequestNewPos(pos);
+        GetEngineControl()->fTransport.RequestNewPos(&tmp);
         return 0;
     }
 }
@@ -877,7 +921,7 @@ void JackClient::TransportStop()
     GetEngineControl()->fTransport.SetCommand(TransportCommandStop);
 }
 
-// Never called concurently with the server
+// Never called concurrently with the server
 // TODO check concurrency with SetSyncCallback
 
 void JackClient::CallSyncCallback()
@@ -943,6 +987,8 @@ void JackClient::OnShutdown(JackShutdownCallback callback, void *arg)
     if (IsActive()) {
         jack_error("You cannot set callbacks on an active client");
     } else {
+        // Shutdown callback will either be an old API version or the new version (with info) 
+        GetClientControl()->fCallback[kShutDownCallback] = (callback != NULL);
         fShutdownArg = arg;
         fShutdown = callback;
     }
@@ -953,6 +999,7 @@ void JackClient::OnInfoShutdown(JackInfoShutdownCallback callback, void *arg)
     if (IsActive()) {
         jack_error("You cannot set callbacks on an active client");
     } else {
+        // Shutdown callback will either be an old API version or the new version (with info)
         GetClientControl()->fCallback[kShutDownCallback] = (callback != NULL);
         fInfoShutdownArg = arg;
         fInfoShutdown = callback;
@@ -996,8 +1043,7 @@ int JackClient::SetInitCallback(JackThreadInitCallback callback, void *arg)
         fInitArg = arg;
         fInit = callback;
         /* make sure that the message buffer thread is initialized too */
-        JackMessageBuffer::fInstance->SetInitCallback(callback, arg);
-        return 0;
+        return JackMessageBuffer::fInstance->SetInitCallback(callback, arg);
     }
 }
 
@@ -1037,8 +1083,9 @@ int JackClient::SetSampleRateCallback(JackSampleRateCallback callback, void *arg
         fSampleRateArg = arg;
         fSampleRate = callback;
         // Now invoke it
-        if (callback)
+        if (callback) {
             callback(GetEngineControl()->fSampleRate, arg);
+        }
         return 0;
     }
 }
@@ -1117,7 +1164,7 @@ int JackClient::SetProcessThread(JackThreadCallback fun, void *arg)
         jack_error("You cannot set callbacks on an active client");
         return -1;
     } else if (fProcess) {
-        jack_error ("A process callback has already been setup, both models cannot be used at the same time!");
+        jack_error("A process callback has already been setup, both models cannot be used at the same time!");
         return -1;
     } else {
         fThreadFun = fun;
@@ -1152,13 +1199,25 @@ int JackClient::SetLatencyCallback(JackLatencyCallback callback, void *arg)
     }
 }
 
+int JackClient::SetPropertyChangeCallback(JackPropertyChangeCallback callback, void *arg)
+{
+    if (IsActive()) {
+        jack_error("You cannot set callbacks on an active client");
+        return -1;
+    } else {
+        fPropertyChangeArg = arg;
+        fPropertyChange = callback;
+        return 0;
+    }
+}
+
 //------------------
 // Internal clients
 //------------------
 
 char* JackClient::GetInternalClientName(int ref)
 {
-    char name_res[JACK_CLIENT_NAME_SIZE + 1];
+    char name_res[JACK_CLIENT_NAME_SIZE+1];
     int result = -1;
     fChannel->GetInternalClientName(GetClientControl()->fRefNum, ref, name_res, &result);
     return (result < 0) ? NULL : strdup(name_res);
@@ -1232,8 +1291,9 @@ int JackClient::SessionReply(jack_session_event_t* ev)
 
     jack_log("JackClient::SessionReply... we are here");
     if (fChannel->IsChannelThread()) {
-        jack_log( "JackClient::SessionReply... in callback reply");
-        fImmediateSessionReply = true;
+        jack_log("JackClient::SessionReply... in callback reply");
+        // OK, immediate reply...
+        fSessionReply = kImmediateSessionReply;
         return 0;
     }
 
@@ -1246,7 +1306,7 @@ int JackClient::SessionReply(jack_session_event_t* ev)
 
 char* JackClient::GetUUIDForClientName(const char* client_name)
 {
-    char uuid_res[JACK_UUID_SIZE];
+    char uuid_res[JACK_UUID_STRING_SIZE];
     int result = -1;
     fChannel->GetUUIDForClientName(GetClientControl()->fRefNum, client_name, uuid_res, &result);
     return (result) ? NULL : strdup(uuid_res);
@@ -1274,5 +1334,17 @@ int JackClient::ClientHasSessionCallback(const char* client_name)
     return result;
 }
 
+//------------------
+// Metadata API
+//------------------
+
+int JackClient::PropertyChangeNotify(jack_uuid_t subject, const char* key, jack_property_change_t change)
+{
+    int result = -1;
+    fChannel->PropertyChangeNotify(subject, key, change, &result);
+    return result;
+}
+
+
 } // end of namespace
 
