@@ -36,6 +36,7 @@ struct JackClientControl;
 \brief Utility class.
 */
 
+PRE_PACKED_STRUCTURE
 template <int SIZE>
 class JackFixedArray
 {
@@ -122,6 +123,7 @@ class JackFixedArray
 \brief Utility class.
 */
 
+PRE_PACKED_STRUCTURE
 template <int SIZE>
 class JackFixedArray1 : public JackFixedArray<SIZE>
 {
@@ -158,6 +160,7 @@ class JackFixedArray1 : public JackFixedArray<SIZE>
 \brief Utility class.
 */
 
+PRE_PACKED_STRUCTURE
 template <int SIZE>
 class JackFixedMatrix
 {
@@ -253,6 +256,7 @@ class JackFixedMatrix
 \brief Utility class.
 */
 
+PRE_PACKED_STRUCTURE
 template <int SIZE>
 class JackLoopFeedback
 {
@@ -366,6 +370,7 @@ class JackLoopFeedback
 \brief For client timing measurements.
 */
 
+PRE_PACKED_STRUCTURE
 struct JackClientTiming
 {
     jack_time_t fSignaledAt;
@@ -396,12 +401,13 @@ struct JackClientTiming
 <UL>
 <LI>The <B>fConnection</B> array contains the list (array line) of connected ports for a given port.
 <LI>The <B>fInputPort</B> array contains the list (array line) of input connected  ports for a given client.
-<LI>The <B>fOutputPort</B> array contains the list (array line) of ouput connected  ports for a given client.
+<LI>The <B>fOutputPort</B> array contains the list (array line) of output connected  ports for a given client.
 <LI>The <B>fConnectionRef</B> array contains the number of ports connected between two clients.
 <LI>The <B>fInputCounter</B> array contains the number of input clients connected to a given for activation purpose.
 </UL>
 */
 
+PRE_PACKED_STRUCTURE
 class SERVER_EXPORT JackConnectionManager
 {
 
