@@ -35,6 +35,7 @@ namespace Jack
 {
 
 JackDebugClient::JackDebugClient(JackClient * client)
+  : JackClient(client->fSynchroTable)
 {
     fTotalPortNumber = 1;       // The total number of port opened and maybe closed. Historical view.
     fOpenPortNumber = 0;        // The current number of opened port.
@@ -58,7 +59,7 @@ JackDebugClient::~JackDebugClient()
     if (fOpenPortNumber != 0)
         *fStream << "!!! WARNING !!! Some ports have not been unregistered ! Incorrect exiting !" << endl;
     if (fIsDeactivated != fIsActivated)
-        *fStream << "!!! ERROR !!! Client seem to not perform symetric activation-deactivation ! (not the same number of activate and deactivate)" << endl;
+        *fStream << "!!! ERROR !!! Client seem to not perform symmetric activation-deactivation ! (not the same number of activate and deactivate)" << endl;
     if (fIsClosed == 0)
         *fStream << "!!! ERROR !!! Client have not been closed with jack_client_close() !" << endl;
 
@@ -78,7 +79,7 @@ JackDebugClient::~JackDebugClient()
     delete fClient;
 }
 
-int JackDebugClient::Open(const char* server_name, const char* name, int uuid, jack_options_t options, jack_status_t* status)
+int JackDebugClient::Open(const char* server_name, const char* name, jack_uuid_t uuid, jack_options_t options, jack_status_t* status)
 {
     int res = fClient->Open(server_name, name, uuid, options, status);
     char provstr[256];
@@ -90,7 +91,7 @@ int JackDebugClient::Open(const char* server_name, const char* name, int uuid, j
     /* Convert it to local time representation. */
     loctime = localtime (&curtime);
     strftime (buffer, 256, "%I-%M", loctime);
-    sprintf(provstr, "JackClientDebug-%s-%s.log", name, buffer);
+    snprintf(provstr, sizeof(provstr), "JackClientDebug-%s-%s.log", name, buffer);
     fStream = new ofstream(provstr, ios_base::ate);
     if (fStream->is_open()) {
         if (res == -1) {
@@ -115,7 +116,11 @@ int JackDebugClient::Close()
 
 void JackDebugClient::CheckClient(const char* function_name) const
 {
+#ifdef WIN32
+    *fStream << "CheckClient : " << function_name << ", calling thread : " << GetCurrentThread() << endl;
+#else
     *fStream << "CheckClient : " << function_name << ", calling thread : " << pthread_self() << endl;
+#endif
 
     if (fIsClosed > 0)  {
         *fStream << "!!! ERROR !!! : Accessing a client '" << fClientName << "' already closed " << "from " << function_name << endl;
@@ -290,13 +295,13 @@ int JackDebugClient::PortDisconnect(jack_port_id_t src)
 {
     CheckClient("PortDisconnect");
     if (!fIsActivated)
-        *fStream << "!!! ERROR !!! : Trying to disconnect port  " << src << " while that client has not been activated !" << endl;
+        *fStream << "!!! ERROR !!! : Trying to disconnect port " << src << " while that client has not been activated !" << endl;
     int res = fClient->PortDisconnect(src);
     int i;
     for (i = (fTotalPortNumber - 1); i >= 0; i--) {             // We search the record into the history
         if (fPortList[i].idport == src) {                               // We found the record in sources
             if (fPortList[i].IsUnregistered != 0)
-                *fStream << "!!! ERROR !!! : Disconnecting port  " << src << " previoulsy unregistered !" << endl;
+                *fStream << "!!! ERROR !!! : Disconnecting port " << src << " previoulsy unregistered !" << endl;
             fPortList[i].IsConnected--;
             *fStream << "Disconnecting port " << src << ". " << endl;
             break;
@@ -313,9 +318,17 @@ int JackDebugClient::PortDisconnect(jack_port_id_t src)
 int JackDebugClient::PortIsMine(jack_port_id_t port_index)
 {
     CheckClient("PortIsMine");
+    *fStream << "JackClientDebug : PortIsMine port_index " << port_index << endl;
     return fClient->PortIsMine(port_index);
 }
 
+int JackDebugClient::PortRename(jack_port_id_t port_index, const char* name)
+{
+    CheckClient("PortRename");
+    *fStream << "JackClientDebug : PortRename port_index " << port_index << "name" << name << endl;
+    return fClient->PortRename(port_index, name);
+}
+
 //--------------------
 // Context management
 //--------------------
@@ -323,6 +336,7 @@ int JackDebugClient::PortIsMine(jack_port_id_t port_index)
 int JackDebugClient::SetBufferSize(jack_nframes_t buffer_size)
 {
     CheckClient("SetBufferSize");
+    *fStream << "JackClientDebug : SetBufferSize buffer_size " << buffer_size << endl;
     return fClient->SetBufferSize(buffer_size);
 }
 
@@ -330,13 +344,19 @@ int JackDebugClient::SetFreeWheel(int onoff)
 {
     CheckClient("SetFreeWheel");
     if (onoff && fFreewheel)
-         *fStream << "!!! ERROR !!! : Freewheel setup seems incorrect : set = ON while FW is already ON " << endl;
+        *fStream << "!!! ERROR !!! : Freewheel setup seems incorrect : set = ON while FW is already ON " << endl;
     if (!onoff && !fFreewheel)
-         *fStream << "!!! ERROR !!! : Freewheel setup seems incorrect : set = OFF while FW is already OFF " << endl;
-    fFreewheel = onoff;
+        *fStream << "!!! ERROR !!! : Freewheel setup seems incorrect : set = OFF while FW is already OFF " << endl;
+    fFreewheel = onoff ? true : false;
     return fClient->SetFreeWheel(onoff);
 }
 
+int JackDebugClient::ComputeTotalLatencies()
+{
+    CheckClient("ComputeTotalLatencies");
+    return fClient->ComputeTotalLatencies();
+}
+
 /*
 ShutDown is called:
 - from the RT thread when Execute method fails
@@ -344,9 +364,10 @@ ShutDown is called:
 (Not needed since the synch object used (Sema of Fifo will fails when server quits... see ShutDown))
 */
 
-void JackDebugClient::ShutDown()
+void JackDebugClient::ShutDown(jack_status_t code, const char* message)
 {
-    fClient->ShutDown();
+    CheckClient("ShutDown");
+    fClient->ShutDown(code, message);
 }
 
 //---------------------
@@ -368,6 +389,7 @@ int JackDebugClient::SetSyncCallback(JackSyncCallback sync_callback, void* arg)
 int JackDebugClient::SetSyncTimeout(jack_time_t timeout)
 {
     CheckClient("SetSyncTimeout");
+    *fStream << "JackClientDebug : SetSyncTimeout timeout " << timeout << endl;
     return fClient->SetSyncTimeout(timeout);
 }
 
@@ -380,6 +402,7 @@ int JackDebugClient::SetTimebaseCallback(int conditional, JackTimebaseCallback t
 void JackDebugClient::TransportLocate(jack_nframes_t frame)
 {
     CheckClient("TransportLocate");
+    *fStream << "JackClientDebug : TransportLocate frame " << frame << endl;
     fClient->TransportLocate(frame);
 }
 
@@ -395,7 +418,7 @@ jack_nframes_t JackDebugClient::GetCurrentTransportFrame()
     return fClient->GetCurrentTransportFrame();
 }
 
-int JackDebugClient::TransportReposition(jack_position_t* pos)
+int JackDebugClient::TransportReposition(const jack_position_t* pos)
 {
     CheckClient("TransportReposition");
     return fClient->TransportReposition(pos);
@@ -435,10 +458,11 @@ int JackDebugClient::TimeCallback(jack_nframes_t nframes, void *arg)
     jack_time_t t1 = GetMicroSeconds();
     int res = client->fProcessTimeCallback(nframes, client->fProcessTimeCallbackArg);
     if (res == 0) {
-    jack_time_t t2 = GetMicroSeconds();
+        jack_time_t t2 = GetMicroSeconds();
         long delta = long((t2 - t1) - client->GetEngineControl()->fPeriodUsecs);
-        if (delta > 0 && !client->fFreewheel)
+        if (delta > 0 && !client->fFreewheel) {
             *client->fStream << "!!! ERROR !!! : Process overload of " << delta << " us" << endl;
+        }
     }
     return res;
 }
@@ -446,9 +470,17 @@ int JackDebugClient::TimeCallback(jack_nframes_t nframes, void *arg)
 int JackDebugClient::SetProcessCallback(JackProcessCallback callback, void *arg)
 {
     CheckClient("SetProcessCallback");
+    
     fProcessTimeCallback = callback;
     fProcessTimeCallbackArg = arg;
-    return fClient->SetProcessCallback(TimeCallback, this);
+        
+    if (callback == NULL) {
+        // Clear the callback...
+        return fClient->SetProcessCallback(callback, arg);
+    } else {
+        // Setup the measuring version...
+        return fClient->SetProcessCallback(TimeCallback, this);
+    }
 }
 
 int JackDebugClient::SetXRunCallback(JackXRunCallback callback, void *arg)
@@ -517,9 +549,16 @@ int JackDebugClient::SetLatencyCallback(JackLatencyCallback callback, void *arg)
     return fClient->SetLatencyCallback(callback, arg);
 }
 
+int JackDebugClient::SetProcessThread(JackThreadCallback fun, void *arg)
+{
+    CheckClient("SetProcessThread");
+    return fClient->SetProcessThread(fun, arg);
+}
+
 jack_session_command_t* JackDebugClient::SessionNotify(const char* target, jack_session_event_type_t type, const char* path)
 {
     CheckClient("SessionNotify");
+    *fStream << "JackClientDebug : SessionNotify target " << target << "type " << type << "path " << path << endl;
     return fClient->SessionNotify(target, type, path);
 }
 
@@ -532,24 +571,28 @@ int JackDebugClient::SessionReply(jack_session_event_t* ev)
 char* JackDebugClient::GetUUIDForClientName(const char* client_name)
 {
     CheckClient("GetUUIDForClientName");
+    *fStream << "JackClientDebug : GetUUIDForClientName client_name " << client_name << endl;
     return fClient->GetUUIDForClientName(client_name);
 }
 
 char* JackDebugClient::GetClientNameByUUID(const char* uuid)
 {
     CheckClient("GetClientNameByUUID");
+    *fStream << "JackClientDebug : GetClientNameByUUID uuid " << uuid << endl;
     return fClient->GetClientNameByUUID(uuid);
 }
 
 int JackDebugClient::ReserveClientName(const char* client_name, const char* uuid)
 {
     CheckClient("ReserveClientName");
+    *fStream << "JackClientDebug : ReserveClientName client_name " << client_name << "uuid " << uuid << endl;
     return fClient->ReserveClientName(client_name, uuid);
 }
 
 int JackDebugClient::ClientHasSessionCallback(const char* client_name)
 {
     CheckClient("ClientHasSessionCallback");
+    *fStream << "JackClientDebug : ClientHasSessionCallback client_name " << client_name << endl;
     return fClient->ClientHasSessionCallback(client_name);
 }
 
