@@ -46,7 +46,7 @@ PortFollower;
 \brief A "decorator" debug client to validate API use.
 */
 
-class SERVER_EXPORT JackDebugClient : public JackClient
+class JackDebugClient : public JackClient
 {
     protected:
 
@@ -59,7 +59,7 @@ class SERVER_EXPORT JackDebugClient : public JackClient
         int fIsDeactivated;
         int fIsClosed;
         bool fFreewheel;
-        char fClientName[JACK_CLIENT_NAME_SIZE + 1];
+        char fClientName[JACK_CLIENT_NAME_SIZE+1];
         JackProcessCallback fProcessTimeCallback;
         void* fProcessTimeCallbackArg;
 
@@ -68,7 +68,7 @@ class SERVER_EXPORT JackDebugClient : public JackClient
         JackDebugClient(JackClient* fTheClient);
         virtual ~JackDebugClient();
 
-        virtual int Open(const char* server_name, const char* name, int uuid, jack_options_t options, jack_status_t* status);
+        virtual int Open(const char* server_name, const char* name, jack_uuid_t uuid, jack_options_t options, jack_status_t* status);
         int Close();
 
         virtual JackGraphManager* GetGraphManager() const;
@@ -83,7 +83,8 @@ class SERVER_EXPORT JackDebugClient : public JackClient
         // Context
         int SetBufferSize(jack_nframes_t buffer_size);
         int SetFreeWheel(int onoff);
-        void ShutDown();
+        int ComputeTotalLatencies();
+        void ShutDown(jack_status_t code, const char* message);
         jack_native_thread_t GetThreadID();
 
         // Port management
@@ -95,6 +96,7 @@ class SERVER_EXPORT JackDebugClient : public JackClient
         int PortDisconnect(jack_port_id_t src);
 
         int PortIsMine(jack_port_id_t port_index);
+        int PortRename(jack_port_id_t port_index, const char* name);
 
         // Transport
         int ReleaseTimebase();
@@ -104,7 +106,7 @@ class SERVER_EXPORT JackDebugClient : public JackClient
         void TransportLocate(jack_nframes_t frame);
         jack_transport_state_t TransportQuery(jack_position_t* pos);
         jack_nframes_t GetCurrentTransportFrame();
-        int TransportReposition(jack_position_t* pos);
+        int TransportReposition(const jack_position_t* pos);
         void TransportStart();
         void TransportStop();
 
@@ -130,6 +132,9 @@ class SERVER_EXPORT JackDebugClient : public JackClient
         int InternalClientLoad(const char* client_name, jack_options_t options, jack_status_t* status, jack_varargs_t* va);
         void InternalClientUnload(int ref, jack_status_t* status);
 
+        // RT Thread
+        int SetProcessThread(JackThreadCallback fun, void *arg);
+
         // Session API
         jack_session_command_t* SessionNotify(const char* target, jack_session_event_type_t type, const char* path);
         int SessionReply(jack_session_event_t* ev);
