@@ -22,6 +22,7 @@
 #include "JackMessageBuffer.h"
 #include "JackGlobals.h"
 #include "JackError.h"
+#include "JackTime.h"
 
 namespace Jack
 {
@@ -29,31 +30,50 @@ namespace Jack
 JackMessageBuffer* JackMessageBuffer::fInstance = NULL;
 
 JackMessageBuffer::JackMessageBuffer()
-    :fInit(NULL),fInitArg(NULL),fThread(this),fInBuffer(0),fOutBuffer(0),fOverruns(0),fRunning(false)
+    :fInit(NULL),
+    fInitArg(NULL),
+    fThread(this),
+    fGuard(),
+    fInBuffer(0),
+    fOutBuffer(0),
+    fOverruns(0),
+    fRunning(false)
 {}
 
 JackMessageBuffer::~JackMessageBuffer()
 {}
 
-void JackMessageBuffer::Start()
+bool JackMessageBuffer::Start()
 {
+    // Before StartSync()...
     fRunning = true;
-    fThread.StartSync();
+    if (fThread.StartSync() == 0) {
+        return true;
+    } else {
+        fRunning = false;
+        return false;
+    }
 }
 
-void JackMessageBuffer::Stop()
+bool JackMessageBuffer::Stop()
 {
     if (fOverruns > 0) {
         jack_error("WARNING: %d message buffer overruns!", fOverruns);
     } else {
         jack_log("no message buffer overruns");
     }
-    fGuard.Lock();
-    fRunning = false;
-    fGuard.Signal();
-    fGuard.Unlock();
-    fThread.Stop();
+
+    if (fGuard.Lock()) {
+        fRunning = false;
+        fGuard.Signal();
+        fGuard.Unlock();
+        fThread.Stop();
+    } else {
+        fThread.Kill();
+    }
+
     Flush();
+    return true;
 }
 
 void JackMessageBuffer::Flush()
@@ -79,38 +99,56 @@ void JackMessageBuffer::AddMessage(int level, const char *message)
 
 bool JackMessageBuffer::Execute()
 {
-    while (fRunning) {
-        fGuard.Lock();
-        fGuard.Wait();
-        /* the client asked for all threads to run a thread
-        initialization callback, which includes us.
-        */
-        if (fInit) {
-            fInit(fInitArg);
-            fInit = NULL;
-            /* and we're done */
-            fGuard.Signal();
+    if (fGuard.Lock()) {
+        while (fRunning) {
+            fGuard.Wait();
+            /* the client asked for all threads to run a thread
+            initialization callback, which includes us.
+            */
+            if (fInit) {
+                fInit(fInitArg);
+                fInit = NULL;
+                /* and we're done */
+                fGuard.Signal();
+            }
+            
+            /* releasing the mutex reduces contention */
+            fGuard.Unlock();
+            Flush();
+            fGuard.Lock();
         }
-        Flush();
         fGuard.Unlock();
+    } else {
+        jack_error("JackMessageBuffer::Execute lock cannot be taken");
     }
+
     return false;
 }
 
-void JackMessageBuffer::Create()
+bool JackMessageBuffer::Create()
 {
     if (fInstance == NULL) {
         fInstance = new JackMessageBuffer();
-        fInstance->Start();
+        if (!fInstance->Start()) {
+            jack_error("JackMessageBuffer::Create cannot start thread");
+            delete fInstance;
+            fInstance = NULL;
+            return false;
+        }
     }
+
+    return true;
 }
 
-void JackMessageBuffer::Destroy()
+bool JackMessageBuffer::Destroy()
 {
     if (fInstance != NULL) {
         fInstance->Stop();
         delete fInstance;
         fInstance = NULL;
+        return true;
+    } else {
+        return false;
     }
 }
 
@@ -124,18 +162,42 @@ void JackMessageBufferAdd(int level, const char *message)
     }
 }
 
-void JackMessageBuffer::SetInitCallback(JackThreadInitCallback callback, void *arg)
+int JackMessageBuffer::SetInitCallback(JackThreadInitCallback callback, void *arg)
 {
-    fGuard.Lock();
-    /* set up the callback */
-    fInitArg = arg;
-    fInit = callback;
-    /* wake msg buffer thread */
-    fGuard.Signal();
-    /* wait for it to be done */
-    fGuard.Wait();
-    /* and we're done */
-    fGuard.Unlock();
+    if (fInstance && callback && fRunning && fGuard.Lock()) {
+        /* set up the callback */
+        fInitArg = arg;
+        fInit = callback;
+        
+    #ifndef WIN32
+        // wake msg buffer thread 
+        fGuard.Signal();
+        // wait for it to be done  
+        fGuard.Wait();
+        // and we're done 
+        fGuard.Unlock();
+    #else
+        /*
+        The condition variable emulation code does not work reliably on Windows (lost signal).
+        So use a "hackish" way to signal/wait for the result.
+        Probably better in the long term : use pthread-win32 (http://sourceware.org/pthreads-win32/`
+        */
+        fGuard.Unlock();
+        int count = 0;
+        while (fInit && ++count < 1000) {
+            /* wake msg buffer thread */
+            fGuard.Signal();
+            JackSleep(1000);
+        }
+        if (count == 1000) {
+            jack_error("JackMessageBuffer::SetInitCallback : signal lost");
+            return -1;
+	}
+    #endif
+        return 0;
+    }
+    jack_error("JackMessageBuffer::SetInitCallback : callback could not be executed");
+    return -1;
 }
 
 };
