@@ -22,7 +22,6 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #include "JackServerGlobals.h"
 #include "JackTime.h"
 #include "JackFreewheelDriver.h"
-#include "JackDummyDriver.h"
 #include "JackThreadedDriver.h"
 #include "JackGlobals.h"
 #include "JackLockedEngine.h"
@@ -34,11 +33,17 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #include "JackInternalClient.h"
 #include "JackError.h"
 #include "JackMessageBuffer.h"
+#include "JackInternalSessionLoader.h"
+
+const char * jack_get_self_connect_mode_description(char mode);
 
 namespace Jack
 {
 
-JackServer::JackServer(bool sync, bool temporary, int timeout, bool rt, int priority, int port_max, bool verbose, jack_timer_type_t clock, const char* server_name)
+//----------------
+// Server control 
+//----------------
+JackServer::JackServer(bool sync, bool temporary, int timeout, bool rt, int priority, int port_max, bool verbose, jack_timer_type_t clock, char self_connect_mode, const char* server_name)
 {
     if (rt) {
         jack_info("JACK server starting in realtime mode with priority %ld", priority);
@@ -46,19 +51,20 @@ JackServer::JackServer(bool sync, bool temporary, int timeout, bool rt, int prio
         jack_info("JACK server starting in non-realtime mode");
     }
 
+    jack_info("self-connect-mode is \"%s\"", jack_get_self_connect_mode_description(self_connect_mode));
+
     fGraphManager = JackGraphManager::Allocate(port_max);
     fEngineControl = new JackEngineControl(sync, temporary, timeout, rt, priority, verbose, clock, server_name);
-    fEngine = new JackLockedEngine(fGraphManager, GetSynchroTable(), fEngineControl);
+    fEngine = new JackLockedEngine(fGraphManager, GetSynchroTable(), fEngineControl, self_connect_mode);
 
     // A distinction is made between the threaded freewheel driver and the
     // regular freewheel driver because the freewheel driver needs to run in
     // threaded mode when freewheel mode is active and needs to run as a slave
     // when freewheel mode isn't active.
-    JackFreewheelDriver *freewheelDriver =
-        new JackFreewheelDriver(fEngine, GetSynchroTable());
+    JackFreewheelDriver* freewheelDriver = new JackFreewheelDriver(fEngine, GetSynchroTable());
     fThreadedFreewheelDriver = new JackThreadedDriver(freewheelDriver);
 
-   fFreewheelDriver = freewheelDriver;
+    fFreewheelDriver = freewheelDriver;
     fDriverInfo = new JackDriverInfo();
     fAudioDriver = NULL;
     fFreewheel = false;
@@ -79,14 +85,16 @@ JackServer::~JackServer()
 int JackServer::Open(jack_driver_desc_t* driver_desc, JSList* driver_params)
 {
     // TODO: move that in reworked JackServerGlobals::Init()
-    JackMessageBuffer::Create();
+    if (!JackMessageBuffer::Create()) {
+        jack_error("Cannot create message buffer");
+    }
 
      if ((fAudioDriver = fDriverInfo->Open(driver_desc, fEngine, GetSynchroTable(), driver_params)) == NULL) {
         jack_error("Cannot initialize driver");
         goto fail_close1;
     }
 
-    if (fChannel.Open(fEngineControl->fServerName, this) < 0) {
+    if (fRequestChannel.Open(fEngineControl->fServerName, this) < 0) {
         jack_error("Server channel open error");
         goto fail_close2;
     }
@@ -120,7 +128,7 @@ fail_close4:
     fEngine->Close();
 
 fail_close3:
-    fChannel.Close();
+    fRequestChannel.Close();
 
 fail_close2:
     fAudioDriver->Close();
@@ -133,38 +141,80 @@ fail_close1:
 int JackServer::Close()
 {
     jack_log("JackServer::Close");
-    fEngine->NotifyQuit();
-    fChannel.Close();
+    fRequestChannel.Close();
     fAudioDriver->Detach();
     fAudioDriver->Close();
     fFreewheelDriver->Close();
     fEngine->Close();
     // TODO: move that in reworked JackServerGlobals::Destroy()
     JackMessageBuffer::Destroy();
+    EndTime();
     return 0;
 }
 
-int JackServer::InternalClientLoad1(const char* client_name, const char* so_name, const char* objet_data, int options, int* int_ref, int uuid, int* status)
+int JackServer::Start()
+{
+    jack_log("JackServer::Start");
+    if (fAudioDriver->Start() < 0) {
+        return -1;
+    }
+    return fRequestChannel.Start();
+}
+
+int JackServer::Stop()
+{
+    jack_log("JackServer::Stop");
+    int res = -1;
+    
+    if (fFreewheel) {
+        if (fThreadedFreewheelDriver) {
+            res = fThreadedFreewheelDriver->Stop();
+        }
+    } else {
+        if (fAudioDriver) {
+            res = fAudioDriver->Stop();
+        }
+    }
+    
+    fEngine->NotifyQuit();
+    fRequestChannel.Stop();
+    fEngine->NotifyFailure(JackFailure | JackServerError, JACK_SERVER_FAILURE);
+    
+    return res;
+}
+
+bool JackServer::IsRunning()
+{
+    jack_log("JackServer::IsRunning");
+    assert(fAudioDriver);
+    return fAudioDriver->IsRunning();
+}
+
+//------------------
+// Internal clients 
+//------------------
+
+int JackServer::InternalClientLoad1(const char* client_name, const char* so_name, const char* objet_data, int options, int* int_ref, jack_uuid_t uuid, int* status)
 {
     JackLoadableInternalClient* client = new JackLoadableInternalClient1(JackServerGlobals::fInstance, GetSynchroTable(), objet_data);
     assert(client);
     return InternalClientLoadAux(client, so_name, client_name, options, int_ref, uuid, status);
  }
 
-int JackServer::InternalClientLoad2(const char* client_name, const char* so_name, const JSList * parameters, int options, int* int_ref, int uuid, int* status)
+int JackServer::InternalClientLoad2(const char* client_name, const char* so_name, const JSList * parameters, int options, int* int_ref, jack_uuid_t uuid, int* status)
 {
     JackLoadableInternalClient* client = new JackLoadableInternalClient2(JackServerGlobals::fInstance, GetSynchroTable(), parameters);
     assert(client);
     return InternalClientLoadAux(client, so_name, client_name, options, int_ref, uuid, status);
 }
 
-int JackServer::InternalClientLoadAux(JackLoadableInternalClient* client, const char* so_name, const char* client_name, int options, int* int_ref, int uuid, int* status)
+int JackServer::InternalClientLoadAux(JackLoadableInternalClient* client, const char* so_name, const char* client_name, int options, int* int_ref, jack_uuid_t uuid, int* status)
 {
     // Clear status
     *status = 0;
 
     // Client object is internally kept in JackEngine
-    if ((client->Init(so_name) < 0) || (client->Open(JACK_DEFAULT_SERVER_NAME, client_name,  uuid, (jack_options_t)options, (jack_status_t*)status) < 0)) {
+    if ((client->Init(so_name) < 0) || (client->Open(JackTools::DefaultServerName(), client_name,  uuid, (jack_options_t)options, (jack_status_t*)status) < 0)) {
         delete client;
         int my_status1 = *status | JackFailure;
         *status = (jack_status_t)my_status1;
@@ -176,31 +226,19 @@ int JackServer::InternalClientLoadAux(JackLoadableInternalClient* client, const
     }
  }
 
-int JackServer::Start()
+//-----------------------
+// Internal session file 
+//-----------------------
+
+int JackServer::LoadInternalSessionFile(const char* file)
 {
-    jack_log("JackServer::Start");
-    if (fAudioDriver->Start() < 0) {
-        return -1;
-    }
-    return fChannel.Start();
+    JackInternalSessionLoader loader(this);
+    return loader.Load(file);
 }
 
-int JackServer::Stop()
-{
-    jack_log("JackServer::Stop");
-    if (fFreewheel) {
-        return fThreadedFreewheelDriver->Stop();
-    } else {
-        return fAudioDriver->Stop();
-    }
-}
-
-bool JackServer::IsRunning()
-{
-    jack_log("JackServer::IsRunning");
-    assert(fAudioDriver);
-    return fAudioDriver->IsRunning();
-}
+//---------------------------
+// From request thread : API 
+//---------------------------
 
 int JackServer::SetBufferSize(jack_nframes_t buffer_size)
 {
@@ -223,13 +261,11 @@ int JackServer::SetBufferSize(jack_nframes_t buffer_size)
     }
 
     if (fAudioDriver->SetBufferSize(buffer_size) == 0) {
-        fFreewheelDriver->SetBufferSize(buffer_size);
         fEngine->NotifyBufferSize(buffer_size);
         return fAudioDriver->Start();
     } else { // Failure: try to restore current value
         jack_error("Cannot SetBufferSize for audio driver, restore current value %ld", current_buffer_size);
         fAudioDriver->SetBufferSize(current_buffer_size);
-        fFreewheelDriver->SetBufferSize(current_buffer_size);
         fAudioDriver->Start();
         // SetBufferSize actually failed, so return an error...
         return -1;
@@ -237,10 +273,10 @@ int JackServer::SetBufferSize(jack_nframes_t buffer_size)
 }
 
 /*
-Freewheel mode is implemented by switching from the (audio + freewheel) driver to the freewheel driver only:
+Freewheel mode is implemented by switching from the (audio [slaves] + freewheel) driver to the freewheel driver only:
 
     - "global" connection state is saved
-    - all audio driver ports are deconnected, thus there is no more dependancies with the audio driver
+    - all audio driver and slaves ports are deconnected, thus there is no more dependencies with the audio driver and slaves
     - the freewheel driver will be synchronized with the end of graph execution : all clients are connected to the freewheel driver
     - the freewheel driver becomes the "master"
 
@@ -258,7 +294,7 @@ int JackServer::SetFreewheel(bool onoff)
         } else {
             fFreewheel = false;
             fThreadedFreewheelDriver->Stop();
-            fGraphManager->Restore(&fConnectionState);   // Restore previous connection state
+            fGraphManager->Restore(&fConnectionState);   // Restore connection state
             fEngine->NotifyFreewheel(onoff);
             fFreewheelDriver->SetMaster(false);
             fAudioDriver->SetMaster(true);
@@ -269,6 +305,15 @@ int JackServer::SetFreewheel(bool onoff)
             fFreewheel = true;
             fAudioDriver->Stop();
             fGraphManager->Save(&fConnectionState);     // Save connection state
+            // Disconnect all slaves
+            std::list<JackDriverInterface*> slave_list = fAudioDriver->GetSlaves();
+            std::list<JackDriverInterface*>::const_iterator it;
+            for (it = slave_list.begin(); it != slave_list.end(); it++) {
+                JackDriver* slave = dynamic_cast<JackDriver*>(*it);
+                assert(slave);
+                fGraphManager->DisconnectAllPorts(slave->GetClientControl()->fRefNum);
+            }
+            // Disconnect master
             fGraphManager->DisconnectAllPorts(fAudioDriver->GetClientControl()->fRefNum);
             fEngine->NotifyFreewheel(onoff);
             fAudioDriver->SetMaster(false);
@@ -280,7 +325,10 @@ int JackServer::SetFreewheel(bool onoff)
     }
 }
 
+//---------------------------
 // Coming from the RT thread
+//---------------------------
+
 void JackServer::Notify(int refnum, int notify, int value)
 {
     switch (notify) {
@@ -290,38 +338,37 @@ void JackServer::Notify(int refnum, int notify, int value)
             break;
 
         case kXRunCallback:
-            fEngine->NotifyXRun(refnum);
+            fEngine->NotifyClientXRun(refnum);
             break;
     }
 }
 
-void JackServer::ClientKill(int refnum)
-{
-    jack_log("JackServer::ClientKill ref = %ld", refnum);
-    if (fEngine->ClientDeactivate(refnum) < 0) {
-        jack_error("JackServer::ClientKill ref = %ld cannot be removed from the graph !!", refnum);
-    }
-    if (fEngine->ClientExternalClose(refnum) < 0) {
-        jack_error("JackServer::ClientKill ref = %ld cannot be closed", refnum);
-    }
-}
-
-//----------------------
+//--------------------
 // Backend management
-//----------------------
+//--------------------
 
 JackDriverInfo* JackServer::AddSlave(jack_driver_desc_t* driver_desc, JSList* driver_params)
 {
     JackDriverInfo* info = new JackDriverInfo();
     JackDriverClientInterface* slave = info->Open(driver_desc, fEngine, GetSynchroTable(), driver_params);
-    if (slave == NULL) {
-        delete info;
-        return NULL;
+    
+    if (!slave) {
+        goto error1;
     }
-    slave->Attach();
+    if (slave->Attach() < 0) {
+        goto error2;
+    }
+    
     slave->SetMaster(false);
     fAudioDriver->AddSlave(slave);
     return info;
+
+error2:
+    slave->Close();
+    
+error1:
+    delete info;
+    return NULL;
 }
 
 void JackServer::RemoveSlave(JackDriverInfo* info)
@@ -334,7 +381,10 @@ void JackServer::RemoveSlave(JackDriverInfo* info)
 
 int JackServer::SwitchMaster(jack_driver_desc_t* driver_desc, JSList* driver_params)
 {
-    /// Remove current master
+    std::list<JackDriverInterface*> slave_list;
+    std::list<JackDriverInterface*>::const_iterator it;
+    
+    // Remove current master
     fAudioDriver->Stop();
     fAudioDriver->Detach();
     fAudioDriver->Close();
@@ -343,15 +393,13 @@ int JackServer::SwitchMaster(jack_driver_desc_t* driver_desc, JSList* driver_par
     JackDriverInfo* info = new JackDriverInfo();
     JackDriverClientInterface* master = info->Open(driver_desc, fEngine, GetSynchroTable(), driver_params);
 
-    if (master == NULL) {
-        delete info;
-        return -1;
+    if (!master) {
+       goto error;
     }
 
     // Get slaves list
-    std::list<JackDriverInterface*> slave_list = fAudioDriver->GetSlaves();
-    std::list<JackDriverInterface*>::const_iterator it;
-
+    slave_list = fAudioDriver->GetSlaves();
+ 
     // Move slaves in new master
     for (it = slave_list.begin(); it != slave_list.end(); it++) {
         JackDriverInterface* slave = *it;
@@ -364,9 +412,22 @@ int JackServer::SwitchMaster(jack_driver_desc_t* driver_desc, JSList* driver_par
     // Activate master
     fAudioDriver = master;
     fDriverInfo = info;
-    fAudioDriver->Attach();
+    
+    if (fAudioDriver->Attach() < 0) {
+        goto error;
+    }
+    
+    // Notify clients of new values
+    fEngine->NotifyBufferSize(fEngineControl->fBufferSize);
+    fEngine->NotifySampleRate(fEngineControl->fSampleRate);
+    
+    // And finally start
     fAudioDriver->SetMaster(true);
     return fAudioDriver->Start();
+
+error:
+    delete info;
+    return -1;
 }
 
 //----------------------
@@ -403,6 +464,5 @@ JackGraphManager* JackServer::GetGraphManager()
     return fGraphManager;
 }
 
-
 } // end of namespace
 
