@@ -74,7 +74,12 @@ jack_get_version(
  *
  */
 const char *
-jack_get_version_string() JACK_OPTIONAL_WEAK_EXPORT;
+jack_get_version_string(void) JACK_OPTIONAL_WEAK_EXPORT;
+
+/**
+ * @defgroup ClientFunctions Creating & manipulating clients
+ * @{
+ */
 
 /**
  * Open an external client session with a JACK server.  This interface
@@ -142,6 +147,29 @@ int jack_client_name_size (void) JACK_OPTIONAL_WEAK_EXPORT;
  */
 char * jack_get_client_name (jack_client_t *client) JACK_OPTIONAL_WEAK_EXPORT;
 
+/**
+ * Get the session ID for a client name.
+ *
+ * The session manager needs this to reassociate a client name to the session_id.
+ *
+ * The caller is responsible for calling jack_free(3) on any non-NULL
+ * returned value.
+ */
+char *jack_get_uuid_for_client_name (jack_client_t *client,
+                                     const char    *client_name) JACK_WEAK_EXPORT;
+
+/**
+ * Get the client name for a session_id.
+ *
+ * In order to snapshot the graph connections, the session manager needs to map
+ * session_ids to client names.
+ *
+ * The caller is responsible for calling jack_free(3) on any non-NULL
+ * returned value.
+ */
+char *jack_get_client_name_by_uuid (jack_client_t *client,
+                                    const char    *client_uuid ) JACK_WEAK_EXPORT;
+
 /**
  * Load an internal client into the Jack server.
  *
@@ -158,7 +186,7 @@ char * jack_get_client_name (jack_client_t *client) JACK_OPTIONAL_WEAK_EXPORT;
  * @param load_name of a shared object file containing the code for
  * the new client.
  *
- * @param load_init an arbitary string passed to the jack_initialize()
+ * @param load_init an arbitrary string passed to the jack_initialize()
  * routine of the new client (may be NULL).
  *
  * @return 0 if successful.
@@ -170,7 +198,7 @@ int jack_internal_client_new (const char *client_name,
 /**
  * Remove an internal client from a JACK server.
  *
- * @deprecated Please use jack_internal_client_load().
+ * @deprecated Please use jack_internal_client_unload().
  */
 void jack_internal_client_close (const char *client_name) JACK_OPTIONAL_WEAK_DEPRECATED_EXPORT;
 
@@ -198,9 +226,9 @@ int jack_get_client_pid (const char *name) JACK_OPTIONAL_WEAK_EXPORT;
 
 /**
  * @return the pthread ID of the thread running the JACK client side
- * code.
+ * real-time code.
  */
-jack_native_thread_t jack_client_thread_id (jack_client_t *) JACK_OPTIONAL_WEAK_EXPORT;
+jack_native_thread_t jack_client_thread_id (jack_client_t *client) JACK_OPTIONAL_WEAK_EXPORT;
 
 /*@}*/
 
@@ -224,7 +252,7 @@ int jack_is_realtime (jack_client_t *client) JACK_OPTIONAL_WEAK_EXPORT;
  *
  * @deprecated Please use jack_cycle_wait() and jack_cycle_signal() functions.
  */
-jack_nframes_t jack_thread_wait (jack_client_t*, int status) JACK_OPTIONAL_WEAK_EXPORT;
+jack_nframes_t jack_thread_wait (jack_client_t *client, int status) JACK_OPTIONAL_WEAK_EXPORT;
 
 /**
  * Wait until this JACK client should process data.
@@ -233,7 +261,7 @@ jack_nframes_t jack_thread_wait (jack_client_t*, int status) JACK_OPTIONAL_WEAK_
  *
  * @return the number of frames of data to process
  */
-    jack_nframes_t jack_cycle_wait (jack_client_t* client) JACK_OPTIONAL_WEAK_EXPORT;
+jack_nframes_t jack_cycle_wait (jack_client_t* client) JACK_OPTIONAL_WEAK_EXPORT;
 
 /**
  * Signal next clients in the graph.
@@ -305,12 +333,17 @@ int jack_set_thread_init_callback (jack_client_t *client,
  * on.  It should be called before jack_client_activate().
  *
  * NOTE: if a client calls this AND jack_on_info_shutdown(), then
- * the event of a client thread shutdown, the callback
+ * in case of a client thread shutdown, the callback
  * passed to this function will not be called, and the one passed to
  * jack_on_info_shutdown() will.
+ *
+ * NOTE: application should typically signal another thread to correctly 
+ * finish cleanup, that is by calling "jack_client_close" 
+ * (since "jack_client_close" cannot be called directly in the context 
+ * of the thread that calls the shutdown callback).
  */
 void jack_on_shutdown (jack_client_t *client,
-                       JackShutdownCallback shutdown_callback, void *arg) JACK_WEAK_EXPORT;
+                       JackShutdownCallback shutdown_callback, void *arg) JACK_OPTIONAL_WEAK_EXPORT;
 
 /**
  * @param client pointer to JACK client structure.
@@ -330,13 +363,17 @@ void jack_on_shutdown (jack_client_t *client,
  * to help more complex clients understand what is going
  * on.  It should be called before jack_client_activate().
  *
- * NOTE: if a client calls this AND jack_on_info_shutdown(), then
- * the event of a client thread shutdown, the callback
- * passed to this function will not be called, and the one passed to
- * jack_on_info_shutdown() will.
+ * NOTE: if a client calls this AND jack_on_shutdown(), then
+ * in case of a client thread shutdown, the callback passed to
+ * jack_on_info_shutdown() will be called.
+ *
+ * NOTE: application should typically signal another thread to correctly 
+ * finish cleanup, that is by calling "jack_client_close" 
+ * (since "jack_client_close" cannot be called directly in the context 
+ * of the thread that calls the shutdown callback).
  */
 void jack_on_info_shutdown (jack_client_t *client,
-                            JackInfoShutdownCallback shutdown_callback, void *arg) JACK_OPTIONAL_WEAK_EXPORT;
+                            JackInfoShutdownCallback shutdown_callback, void *arg) JACK_WEAK_EXPORT;
 
 /**
  * Tell the Jack server to call @a process_callback whenever there is
@@ -366,7 +403,7 @@ int jack_set_process_callback (jack_client_t *client,
  * callback will be non-zero if JACK is entering freewheel
  * mode, and zero otherwise.
  *
- * All "notification events" are received in a seperated non RT thread,
+ * All "notification events" are received in a separated non RT thread,
  * the code in the supplied function does not need to be
  * suitable for real-time execution.
  *
@@ -385,7 +422,7 @@ int jack_set_freewheel_callback (jack_client_t *client,
  * change.  Clients that depend on knowing the buffer size must supply
  * a @a bufsize_callback before activating themselves.
  *
- * All "notification events" are received in a seperated non RT thread,
+ * All "notification events" are received in a separated non RT thread,
  * the code in the supplied function does not need to be
  * suitable for real-time execution.
  *
@@ -406,7 +443,7 @@ int jack_set_buffer_size_callback (jack_client_t *client,
  * Tell the Jack server to call @a srate_callback whenever the system
  * sample rate changes.
  *
- * All "notification events" are received in a seperated non RT thread,
+ * All "notification events" are received in a separated non RT thread,
  * the code in the supplied function does not need to be
  * suitable for real-time execution.
  *
@@ -423,7 +460,7 @@ int jack_set_sample_rate_callback (jack_client_t *client,
  * Tell the JACK server to call @a client_registration_callback whenever a
  * client is registered or unregistered, passing @a arg as a parameter.
  *
- * All "notification events" are received in a seperated non RT thread,
+ * All "notification events" are received in a separated non RT thread,
  * the code in the supplied function does not need to be
  * suitable for real-time execution.
  *
@@ -432,15 +469,15 @@ int jack_set_sample_rate_callback (jack_client_t *client,
  *
  * @return 0 on success, otherwise a non-zero error code
  */
-int jack_set_client_registration_callback (jack_client_t *,
-                       JackClientRegistrationCallback
-                                           registration_callback, void *arg) JACK_OPTIONAL_WEAK_EXPORT;
+int jack_set_client_registration_callback (jack_client_t *client,
+                                            JackClientRegistrationCallback
+                                            registration_callback, void *arg) JACK_OPTIONAL_WEAK_EXPORT;
 
 /**
  * Tell the JACK server to call @a registration_callback whenever a
  * port is registered or unregistered, passing @a arg as a parameter.
  *
- * All "notification events" are received in a seperated non RT thread,
+ * All "notification events" are received in a separated non RT thread,
  * the code in the supplied function does not need to be
  * suitable for real-time execution.
  *
@@ -449,7 +486,7 @@ int jack_set_client_registration_callback (jack_client_t *,
  *
  * @return 0 on success, otherwise a non-zero error code
  */
- int jack_set_port_registration_callback (jack_client_t *,
+ int jack_set_port_registration_callback (jack_client_t *client,
                                           JackPortRegistrationCallback
                                           registration_callback, void *arg) JACK_OPTIONAL_WEAK_EXPORT;
 
@@ -457,7 +494,7 @@ int jack_set_client_registration_callback (jack_client_t *,
  * Tell the JACK server to call @a connect_callback whenever a
  * port is connected or disconnected, passing @a arg as a parameter.
  *
- * All "notification events" are received in a seperated non RT thread,
+ * All "notification events" are received in a separated non RT thread,
  * the code in the supplied function does not need to be
  * suitable for real-time execution.
  *
@@ -466,7 +503,7 @@ int jack_set_client_registration_callback (jack_client_t *,
  *
  * @return 0 on success, otherwise a non-zero error code
  */
-int jack_set_port_connect_callback (jack_client_t *,
+int jack_set_port_connect_callback (jack_client_t *client,
                                     JackPortConnectCallback
                                     connect_callback, void *arg) JACK_OPTIONAL_WEAK_EXPORT;
 
@@ -474,7 +511,7 @@ int jack_set_port_connect_callback (jack_client_t *,
  * Tell the JACK server to call @a rename_callback whenever a
  * port is renamed, passing @a arg as a parameter.
  *
- * All "notification events" are received in a seperated non RT thread,
+ * All "notification events" are received in a separated non RT thread,
  * the code in the supplied function does not need to be
  * suitable for real-time execution.
  *
@@ -483,7 +520,7 @@ int jack_set_port_connect_callback (jack_client_t *,
  *
  * @return 0 on success, otherwise a non-zero error code
  */
-int jack_set_port_rename_callback (jack_client_t *,
+int jack_set_port_rename_callback (jack_client_t *client,
                                    JackPortRenameCallback
                                    rename_callback, void *arg) JACK_OPTIONAL_WEAK_EXPORT;
 
@@ -491,7 +528,7 @@ int jack_set_port_rename_callback (jack_client_t *,
  * Tell the JACK server to call @a graph_callback whenever the
  * processing graph is reordered, passing @a arg as a parameter.
  *
- * All "notification events" are received in a seperated non RT thread,
+ * All "notification events" are received in a separated non RT thread,
  * the code in the supplied function does not need to be
  * suitable for real-time execution.
  *
@@ -500,7 +537,7 @@ int jack_set_port_rename_callback (jack_client_t *,
  *
  * @return 0 on success, otherwise a non-zero error code
  */
-int jack_set_graph_order_callback (jack_client_t *,
+int jack_set_graph_order_callback (jack_client_t *client,
                                    JackGraphOrderCallback graph_callback,
                                    void *) JACK_OPTIONAL_WEAK_EXPORT;
 
@@ -508,7 +545,7 @@ int jack_set_graph_order_callback (jack_client_t *,
  * Tell the JACK server to call @a xrun_callback whenever there is a
  * xrun, passing @a arg as a parameter.
  *
- * All "notification events" are received in a seperated non RT thread,
+ * All "notification events" are received in a separated non RT thread,
  * the code in the supplied function does not need to be
  * suitable for real-time execution.
  *
@@ -517,7 +554,7 @@ int jack_set_graph_order_callback (jack_client_t *,
  *
  * @return 0 on success, otherwise a non-zero error code
  */
-int jack_set_xrun_callback (jack_client_t *,
+int jack_set_xrun_callback (jack_client_t *client,
                             JackXRunCallback xrun_callback, void *arg) JACK_OPTIONAL_WEAK_EXPORT;
 
 /*@}*/
@@ -568,6 +605,10 @@ int jack_set_xrun_callback (jack_client_t *,
  * Clients that do not meet any of those conditions SHOULD
  * register a latency callback.
  *
+ * Another case is when a client wants to use
+ * @ref jack_port_get_latency_range(), which only returns meaninful
+ * values when ports get connected and latency values change.
+ *
  * See the documentation for  @ref jack_port_set_latency_range()
  * on how the callback should operate. Remember that the @a mode
  * argument given to the latency callback will need to be
@@ -575,7 +616,7 @@ int jack_set_xrun_callback (jack_client_t *,
  *
  * @return 0 on success, otherwise a non-zero error code
  */
-int jack_set_latency_callback (jack_client_t *,
+int jack_set_latency_callback (jack_client_t *client,
 			       JackLatencyCallback latency_callback,
 			       void *) JACK_WEAK_EXPORT;
 /*@}*/
@@ -713,7 +754,7 @@ jack_port_t * jack_port_register (jack_client_t *client,
  *
  * @return 0 on success, otherwise a non-zero error code
  */
-int jack_port_unregister (jack_client_t *, jack_port_t *) JACK_OPTIONAL_WEAK_EXPORT;
+int jack_port_unregister (jack_client_t *client, jack_port_t *port) JACK_OPTIONAL_WEAK_EXPORT;
 
 /**
  * This returns a pointer to the memory area associated with the
@@ -732,9 +773,16 @@ int jack_port_unregister (jack_client_t *, jack_port_t *) JACK_OPTIONAL_WEAK_EXP
  * address from there.
  *
  * Caching output ports is DEPRECATED in Jack 2.0, due to some new optimization (like "pipelining").
- * Port buffers have to be retrieved in each callback for proper functionning.
+ * Port buffers have to be retrieved in each callback for proper functioning.
  */
-void * jack_port_get_buffer (jack_port_t *, jack_nframes_t) JACK_OPTIONAL_WEAK_EXPORT;
+void * jack_port_get_buffer (jack_port_t *port, jack_nframes_t) JACK_OPTIONAL_WEAK_EXPORT;
+
+/**
+ * @return the UUID of the jack_port_t
+ *
+ * @see jack_uuid_to_string() to convert into a string representation
+ */
+jack_uuid_t jack_port_uuid (const jack_port_t *port) JACK_OPTIONAL_WEAK_EXPORT;
 
 /**
  * @return the full name of the jack_port_t (including the @a
@@ -771,7 +819,7 @@ jack_port_type_id_t jack_port_type_id (const jack_port_t *port) JACK_OPTIONAL_WE
 /**
  * @return TRUE if the jack_port_t belongs to the jack_client_t.
  */
-int jack_port_is_mine (const jack_client_t *, const jack_port_t *port) JACK_OPTIONAL_WEAK_EXPORT;
+int jack_port_is_mine (const jack_client_t *client, const jack_port_t *port) JACK_OPTIONAL_WEAK_EXPORT;
 
 /**
  * @return number of connections to or from @a port.
@@ -793,7 +841,7 @@ int jack_port_connected_to (const jack_port_t *port,
  * @return a null-terminated array of full port names to which the @a
  * port is connected.  If none, returns NULL.
  *
- * The caller is responsible for calling jack_free(3) on any non-NULL
+ * The caller is responsible for calling jack_free() on any non-NULL
  * returned value.
  *
  * @param port locally owned jack_port_t pointer.
@@ -806,7 +854,7 @@ const char ** jack_port_get_connections (const jack_port_t *port) JACK_OPTIONAL_
  * @return a null-terminated array of full port names to which the @a
  * port is connected.  If none, returns NULL.
  *
- * The caller is responsible for calling jack_free(3) on any non-NULL
+ * The caller is responsible for calling jack_free() on any non-NULL
  * returned value.
  *
  * This differs from jack_port_get_connections() in two important
@@ -843,13 +891,28 @@ int jack_port_tie (jack_port_t *src, jack_port_t *dst) JACK_OPTIONAL_WEAK_DEPREC
 int jack_port_untie (jack_port_t *port) JACK_OPTIONAL_WEAK_DEPRECATED_EXPORT;
 
 /**
+ * \bold THIS FUNCTION IS DEPRECATED AND SHOULD NOT BE USED IN
+ *  NEW JACK CLIENTS
+ *
  * Modify a port's short name.  May be called at any time.  If the
  * resulting full name (including the @a "client_name:" prefix) is
  * longer than jack_port_name_size(), it will be truncated.
  *
  * @return 0 on success, otherwise a non-zero error code.
  */
-int jack_port_set_name (jack_port_t *port, const char *port_name) JACK_OPTIONAL_WEAK_EXPORT;
+int jack_port_set_name (jack_port_t *port, const char *port_name) JACK_OPTIONAL_WEAK_DEPRECATED_EXPORT;
+
+/**
+ * Modify a port's short name.  May NOT be called from a callback handling a server event.
+ * If the resulting full name (including the @a "client_name:" prefix) is
+ * longer than jack_port_name_size(), it will be truncated.
+ *
+ * @return 0 on success, otherwise a non-zero error code.
+ *
+ * This differs from jack_port_set_name() by triggering PortRename notifications to
+ * clients that have registered a port rename handler.
+ */
+int jack_port_rename (jack_client_t* client, jack_port_t *port, const char *port_name) JACK_OPTIONAL_WEAK_EXPORT;
 
 /**
  * Set @a alias as an alias for @a port.  May be called at any time.
@@ -931,7 +994,7 @@ int jack_port_monitoring_input (jack_port_t *port) JACK_OPTIONAL_WEAK_EXPORT;
  * @return 0 on success, EEXIST if the connection is already made,
  * otherwise a non-zero error code
  */
-int jack_connect (jack_client_t *,
+int jack_connect (jack_client_t *client,
                   const char *source_port,
                   const char *destination_port) JACK_OPTIONAL_WEAK_EXPORT;
 
@@ -948,7 +1011,7 @@ int jack_connect (jack_client_t *,
  *
  * @return 0 on success, otherwise a non-zero error code
  */
-int jack_disconnect (jack_client_t *,
+int jack_disconnect (jack_client_t *client,
                      const char *source_port,
                      const char *destination_port) JACK_OPTIONAL_WEAK_EXPORT;
 
@@ -961,7 +1024,7 @@ int jack_disconnect (jack_client_t *,
  * while generic connection clients (e.g. patchbays) would use
  * jack_disconnect().
  */
-int jack_port_disconnect (jack_client_t *, jack_port_t *) JACK_OPTIONAL_WEAK_EXPORT;
+int jack_port_disconnect (jack_client_t *client, jack_port_t *port) JACK_OPTIONAL_WEAK_EXPORT;
 
 /**
  * @return the maximum number of characters in a full JACK port name
@@ -1005,7 +1068,7 @@ size_t jack_port_type_get_buffer_size (jack_client_t *client, const char *port_t
  * latency values associated with them, both measured in frames:
  *
  * <b>capture latency</b>: how long since the data read from
- *                  the buffer of a port arrived at at
+ *                  the buffer of a port arrived at
  *                  a port marked with JackPortIsTerminal.
  *                  The data will have come from the "outside
  *                  world" if the terminal port is also
@@ -1023,7 +1086,7 @@ size_t jack_port_type_get_buffer_size (jack_client_t *client, const char *port_t
  * expressed a min/max pair.
  *
  * In most common setups, the minimum and maximum latency
- * are the same, but this design accomodates more complex
+ * are the same, but this design accommodates more complex
  * routing, and allows applications (and thus users) to
  * detect cases where routing is creating an anomalous
  * situation that may either need fixing or more
@@ -1052,7 +1115,7 @@ size_t jack_port_type_get_buffer_size (jack_client_t *client, const char *port_t
  * be replaced by a latency callback that calls @ref
  * jack_port_set_latency_range().
  */
-void jack_port_set_latency (jack_port_t *, jack_nframes_t) JACK_OPTIONAL_WEAK_DEPRECATED_EXPORT;
+void jack_port_set_latency (jack_port_t *port, jack_nframes_t) JACK_OPTIONAL_WEAK_DEPRECATED_EXPORT;
 
 /**
  * return the latency range defined by @a mode for
@@ -1060,8 +1123,11 @@ void jack_port_set_latency (jack_port_t *, jack_nframes_t) JACK_OPTIONAL_WEAK_DE
  *
  * See @ref LatencyFunctions for the definition of each latency value.
  *
- * This is normally used in the LatencyCallback.
- * and therefor safe to execute from callbacks.
+ * This function is best used from callbacks, specifically the latency callback.
+ * Before a port is connected, this returns the default latency: zero.
+ * Therefore it only makes sense to call jack_port_get_latency_range() when
+ * the port is connected, and that gets signalled by the latency callback.
+ * See @ref jack_set_latency_callback() for details.
  */
 void jack_port_get_latency_range (jack_port_t *port, jack_latency_callback_mode_t mode, jack_latency_range_t *range) JACK_WEAK_EXPORT;
 
@@ -1147,7 +1213,7 @@ void jack_port_set_latency_range (jack_port_t *port, jack_latency_callback_mode_
  * @return zero for successful execution of the request. non-zero
  *         otherwise.
  */
-int jack_recompute_total_latencies (jack_client_t*) JACK_OPTIONAL_WEAK_EXPORT;
+int jack_recompute_total_latencies (jack_client_t *client) JACK_OPTIONAL_WEAK_EXPORT;
 
 /**
  * @return the time (in frames) between data being available or
@@ -1176,7 +1242,7 @@ jack_nframes_t jack_port_get_latency (jack_port_t *port) JACK_OPTIONAL_WEAK_DEPR
  * be replaced by jack_port_get_latency_range() in any existing
  * use cases.
  */
-jack_nframes_t jack_port_get_total_latency (jack_client_t *,
+jack_nframes_t jack_port_get_total_latency (jack_client_t *client,
 					    jack_port_t *port) JACK_OPTIONAL_WEAK_DEPRECATED_EXPORT;
 
 /**
@@ -1215,12 +1281,12 @@ int jack_recompute_total_latency (jack_client_t*, jack_port_t* port) JACK_OPTION
  * If zero, no selection based on flags will be carried out.
  *
  * @return a NULL-terminated array of ports that match the specified
- * arguments.  The caller is responsible for calling jack_free(3) any
+ * arguments.  The caller is responsible for calling jack_free() any
  * non-NULL returned value.
  *
  * @see jack_port_name_size(), jack_port_type_size()
  */
-const char ** jack_get_ports (jack_client_t *,
+const char ** jack_get_ports (jack_client_t *client,
                               const char *port_name_pattern,
                               const char *type_name_pattern,
                               unsigned long flags) JACK_OPTIONAL_WEAK_EXPORT;
@@ -1230,7 +1296,7 @@ const char ** jack_get_ports (jack_client_t *,
  *
  * @see jack_port_name_size()
  */
-jack_port_t * jack_port_by_name (jack_client_t *, const char *port_name) JACK_OPTIONAL_WEAK_EXPORT;
+jack_port_t * jack_port_by_name (jack_client_t *client, const char *port_name) JACK_OPTIONAL_WEAK_EXPORT;
 
 /**
  * @return address of the jack_port_t of a @a port_id.
@@ -1279,6 +1345,55 @@ jack_nframes_t jack_frame_time (const jack_client_t *) JACK_OPTIONAL_WEAK_EXPORT
  */
 jack_nframes_t jack_last_frame_time (const jack_client_t *client) JACK_OPTIONAL_WEAK_EXPORT;
 
+/**
+ * This function may only be used from the process callback.
+ * It provides the internal cycle timing information as used by
+ * most of the other time related functions. This allows the
+ * caller to map between frame counts and microseconds with full
+ * precision (i.e. without rounding frame times to integers),
+ * and also provides e.g. the microseconds time of the start of
+ * the current cycle directly (it has to be computed otherwise).
+ *
+ * If the return value is zero, the following information is
+ * provided in the variables pointed to by the arguments:
+ *
+ * current_frames: the frame time counter at the start of the
+ *                 current cycle, same as jack_last_frame_time().
+ * current_usecs:  the microseconds time at the start of the
+ *                 current cycle.
+ * next_usecs:     the microseconds time of the start of the next
+ *                 next cycle as computed by the DLL.
+ * period_usecs:   the current best estimate of the period time in
+ *                  microseconds.
+ *
+ * NOTES:
+ * 
+ * Because of the types used, all the returned values except period_usecs
+ * are unsigned. In computations mapping between frames and microseconds
+ * *signed* differences are required. The easiest way is to compute those
+ * separately and assign them to the appropriate signed variables,
+ * int32_t for frames and int64_t for usecs. See the implementation of
+ * jack_frames_to_time() and Jack_time_to_frames() for an example.
+ * 
+ * Unless there was an xrun, skipped cycles, or the current cycle is the
+ * first after freewheeling or starting Jack, the value of current_usecs
+ * will always be the value of next_usecs of the previous cycle.
+ *
+ * The value of period_usecs will in general NOT be exactly equal to
+ * the difference of next_usecs and current_usecs. This is because to
+ * ensure stability of the DLL and continuity of the mapping, a fraction
+ * of the loop error must be included in next_usecs. For an accurate
+ * mapping between frames and microseconds, the difference of next_usecs
+ * and current_usecs should be used, and not period_usecs.
+ *
+ * @return zero if OK, non-zero otherwise.
+ */
+int jack_get_cycle_times(const jack_client_t *client,
+                        jack_nframes_t *current_frames,
+                        jack_time_t    *current_usecs,
+                        jack_time_t    *next_usecs,
+                        float          *period_usecs) JACK_OPTIONAL_WEAK_EXPORT;
+                  
 /**
  * @return the estimated time in microseconds of the specified frame time
  */
@@ -1295,7 +1410,7 @@ jack_nframes_t jack_time_to_frames(const jack_client_t *client, jack_time_t) JAC
  *
  * The value returned is guaranteed to be monotonic, but not linear.
  */
-jack_time_t jack_get_time() JACK_OPTIONAL_WEAK_EXPORT;
+jack_time_t jack_get_time(void) JACK_OPTIONAL_WEAK_EXPORT;
 
 /*@}*/
 
@@ -1346,10 +1461,11 @@ void jack_set_info_function (void (*func)(const char *)) JACK_OPTIONAL_WEAK_EXPO
 
 /**
  * The free function to be used on memory returned by jack_port_get_connections,
- * jack_port_get_all_connections and jack_get_ports functions.
+ * jack_port_get_all_connections, jack_get_ports and jack_get_internal_client_name functions.
  * This is MANDATORY on Windows when otherwise all nasty runtime version related crashes can occur.
  * Developers are strongly encouraged to use this function instead of the standard "free" function in new code.
  *
+ * @param ptr the memory pointer to be deallocated.
  */
 void jack_free(void* ptr) JACK_OPTIONAL_WEAK_EXPORT;
 
