@@ -53,6 +53,7 @@ static jackctl_internal_t * jackctl_server_get_internal(jackctl_server_t *server
     return NULL;
 }
 
+#if 0
 static jackctl_parameter_t *
 jackctl_get_parameter(
     const JSList * parameters_list,
@@ -70,6 +71,7 @@ jackctl_get_parameter(
 
     return NULL;
 }
+#endif
 
 static void print_value(union jackctl_parameter_value value, jackctl_param_type_t type)
 {
@@ -142,7 +144,7 @@ int main(int argc, char *argv[])
     const JSList * drivers;
     const JSList * internals;
     const JSList * node_ptr;
-    sigset_t signals;
+    jackctl_sigmask_t * sigmask;
     int opt, option_index;
     const char* driver_name = "dummy";
     const char* client_name = "audioadapter";
@@ -151,9 +153,10 @@ int main(int argc, char *argv[])
 	struct option long_options[] = {
 		{"driver", 1, 0, 'd'},
 		{"client", 1, 0, 'c'},
+        {0, 0, 0, 0}
 	};
 
- 	while ((opt = getopt_long (argc, argv, options, long_options, &option_index)) != EOF) {
+ 	while ((opt = getopt_long (argc, argv, options, long_options, &option_index)) != -1) {
 		switch (opt) {
 			case 'd':
 				driver_name = optarg;
@@ -167,7 +170,7 @@ int main(int argc, char *argv[])
 		}
 	}
 
-	server = jackctl_server_create(NULL, NULL);
+	server = jackctl_server_create2(NULL, NULL, NULL);
     parameters = jackctl_server_get_parameters(server);
 
     /*
@@ -232,8 +235,8 @@ int main(int argc, char *argv[])
 
     */
 
-    signals = jackctl_setup_signals(0);
-    jackctl_wait_signals(signals);
+    sigmask = jackctl_setup_signals(0);
+    jackctl_wait_signals(sigmask);
     jackctl_server_stop(server);
     jackctl_server_close(server);
     jackctl_server_destroy(server);
