@@ -69,12 +69,15 @@ static inline char CAS(volatile UInt32 value, UInt32 newvalue, volatile void* ad
 
 #endif
 
+
+
+
 #if !defined(__i386__) && !defined(__x86_64__)  && !defined(__PPC__)
-#warning using builtin gcc (version > 4.1) atomic
+
 
 static inline char CAS(volatile UInt32 value, UInt32 newvalue, volatile void* addr)
 {
-    return __sync_bool_compare_and_swap (&addr, value, newvalue);
+    return __sync_bool_compare_and_swap ((UInt32*)addr, value, newvalue);
 }
 #endif
 
