@@ -514,7 +514,7 @@ int JackOSSDriver::OpenAux()
     // In duplex mode, check that input and output use the same buffer size
     /*
 
-    10/02/09 : desactivated for now, needs more check (only needed when *same* device is used for input and output ??)
+    10/02/09 : deactivated for now, needs more check (only needed when *same* device is used for input and output ??)
 
     if ((fRWMode & kRead) && (fRWMode & kWrite) && (fInputBufferSize != fOutputBufferSize)) {
        jack_error("JackOSSDriver::OpenAux input and output buffer size are not the same!!");
@@ -625,7 +625,7 @@ int JackOSSDriver::Write()
         fFirstCycle = false;
         memset(fOutputBuffer, 0, fOutputBufferSize);
 
-        // Prefill ouput buffer
+        // Prefill output buffer
         for (int i = 0; i < fNperiods; i++) {
            count = ::write(fOutFD, fOutputBuffer, fOutputBufferSize);
            if (count < (int)fOutputBufferSize) {
@@ -776,7 +776,7 @@ SERVER_EXPORT jack_driver_desc_t* driver_get_descriptor()
     return desc;
 }
 
-EXPORT Jack::JackDriverClientInterface* driver_initialize(Jack::JackLockedEngine* engine, Jack::JackSynchro* table, const JSList* params)
+SERVER_EXPORT Jack::JackDriverClientInterface* driver_initialize(Jack::JackLockedEngine* engine, Jack::JackSynchro* table, const JSList* params)
 {
     int bits = OSS_DRIVER_DEF_BITS;
     jack_nframes_t srate = OSS_DRIVER_DEF_FS;
